// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/ChargedLeptons.hh"
#include "Rivet/Projections/NeutralFinalState.hh"

namespace Rivet {

  class CMS_2015_I1346843 : public Analysis {
  public:
    // Constructor
    CMS_2015_I1346843() : Analysis("CMS_2015_I1346843") {
    }

  public:
    // Book histograms and initialise projections before the run
    void init() {
      FinalState pFS(-2.4, 2.4, 9.*GeV);
      addProjection(pFS, "FinalState");

      ChargedLeptons pFS_mu(pFS);
      addProjection(pFS_mu, "FinalState_mu");

      NeutralFinalState pFS_neu(-2.4, 2.4, 5.*GeV);
      addProjection(pFS_neu, "FinalState_neu");

      _hist_pho_et           = bookHisto1D(1, 1, 1);  // photon transverse energy
      _hist_pho_et_wide      = bookHisto1D(1, 2, 1);  // photon transverse energy (0.5 < dr < 3.0)
      _hist_pho_et_close     = bookHisto1D(1, 3, 1);  // photon transverse energy (0.05 < dr < 0.5)
      _hist_pho_et_lqt       = bookHisto1D(1, 4, 1);  // photon transverse energy (q_T < 10)
      _hist_pho_et_hqt       = bookHisto1D(1, 5, 1);  // photon transverse energy (q_T > 50)
      _hist_pho_dr           = bookHisto1D(2, 1, 1);  // delta_R
      _hist_pho_dr_lqt       = bookHisto1D(2, 2, 1);  // delta_R (q_T < 10)
      _hist_pho_dr_hqt       = bookHisto1D(2, 3, 1);  // delta_R  (q_T > 50)
    }


    // Perform the per-event analysis
    void analyze(const Event& event) {
      const double weight = event.weight();
      const NeutralFinalState& pFS_neu = applyProjection<NeutralFinalState>(event, "FinalState_neu");
      const ChargedLeptons& pFS_mu = applyProjection<ChargedLeptons>(event, "FinalState_mu");

      // at least 2 muons in event
      if (pFS_mu.size() > 1) {
        // order the leptons by pT
        const Particles finalLep = pFS_mu.particlesByPt();
        // analysis cuts on muons
        if (finalLep[0].pT() > 31. && finalLep[1].pT() > 9. && finalLep[0].charge()*finalLep[1].charge() == -1){
          float mass = (finalLep[0].mom() + finalLep[1].mom()).mass();
          // analysis cuts on invariant mass
          if (mass > 30. && mass < 87.){
            // get all the neutral final state particles
            const Particles finalNeu = pFS_neu.particlesByPt();
            // this bool is to make sure we only look at the lead photon
            bool leadPho = true;
            // loop over neutrals
            for (Particles::const_iterator part = finalNeu.begin(); part != finalNeu.end(); ++part) {
              // make sure it's a photon and that it's from the hard process
              if (part->pid() == 22 && !part->fromDecay() && part->isStable()){
                // and in the analysis acceptance
                if ((part->abseta() < 1.4 || part->abseta() > 1.6) && part->abseta() < 2.4) {
                  // calculate the deltaR in eta,phi space with respect to the closest lepton
                  float dR = std::min(deltaR(part->eta(), part->phi(),
                                             finalLep[0].eta(),finalLep[0].phi()),
                                      deltaR(part->eta(), part->phi(),
                                             finalLep[1].eta(), finalLep[1].phi()));
                  // photon must be within analysis cuts and be the leading photon
                  if (dR > 0.05 && dR <= 3.0 && leadPho){
                    leadPho = false;
                    // caculate the three-body (mu,mu,gamma) transverse momentum
                    float qT = (finalLep[0].mom() + finalLep[1].mom() + part->mom()).pT();
                    // fill the analysis histograms
                    _hist_pho_et->fill(part->pT() / GeV, weight);
                    if (dR <= 0.5) _hist_pho_et_close->fill(part->pT() / GeV, weight);
                    if (dR > 0.5) _hist_pho_et_wide->fill(part->pT() / GeV, weight);
                    if (qT < 10.) _hist_pho_et_lqt->fill(part->pT() / GeV, weight);
                    if (qT > 50.) _hist_pho_et_hqt->fill(part->pT() / GeV, weight);
                    _hist_pho_dr->fill(dR, weight);
                    if (qT < 10.) _hist_pho_dr_lqt->fill(dR, weight);
                    if (qT > 50.) _hist_pho_dr_hqt->fill(dR, weight);
                  }  // end photon dr cuts
                } // end photon acceptance cuts
              } // end photon pid and prompt check
            } // end loop over neutral final state particles
          } // end di-lepton mass cut
        } // end muon acceptance cuts
      } // end muon collection size check

    } // end analyze()


    /// Normalise histograms etc., after the run
    void finalize() {
      scale(_hist_pho_et, crossSection() / sumOfWeights());
      scale(_hist_pho_et_wide, crossSection() / sumOfWeights());
      scale(_hist_pho_et_close, crossSection() / sumOfWeights());
      scale(_hist_pho_et_lqt, crossSection() / sumOfWeights());
      scale(_hist_pho_et_hqt, crossSection() / sumOfWeights());
      scale(_hist_pho_dr, crossSection() / sumOfWeights());
      scale(_hist_pho_dr_lqt, crossSection() / sumOfWeights());
      scale(_hist_pho_dr_hqt, crossSection() / sumOfWeights());
    }

  private:
    Histo1DPtr  _hist_pho_et;
    Histo1DPtr  _hist_pho_et_wide;
    Histo1DPtr  _hist_pho_et_close;
    Histo1DPtr  _hist_pho_et_lqt;
    Histo1DPtr  _hist_pho_et_hqt;
    Histo1DPtr  _hist_pho_dr;
    Histo1DPtr  _hist_pho_dr_lqt;
    Histo1DPtr  _hist_pho_dr_hqt;
  };

  DECLARE_RIVET_PLUGIN(CMS_2015_I1346843);
}
